@@ -35,7 +35,7 @@ FTDCParser::parseInfoChunk(const bson_t *bson) {
     auto json = bson_as_json(bson, &length);
     BOOST_LOG_TRIVIAL(debug) << json;
 
-    this->metadata.push_back(std::string(json));
+    metadata.emplace_back(json);
 
     bson_free(json);
 
@@ -55,24 +55,22 @@ FTDCParser::parseInfoChunk(const bson_t *bson) {
         // only once
         logChunkMetrics = FALSE;
 
-        // Decompress and check sizes
-        if (chunk->Decompress() > 0) {
-            auto data = chunk->getUncompressedData();
-
-            // We construct the metrics. This are name and first value only since deltasInChunk have not been read.
-            chunk->ConstructMetrics(data);
-
-            // Get actual values
-            chunk->ReadVariableSizedInts();
-            chunk->setTimestampLimits();
+        if (dataSet->getLazyParsing()) {
             dataSet->addChunk(chunk);
         } else {
-            delete chunk;
-            BOOST_LOG_TRIVIAL(error) << "Could not decompress chunk! "  << "Task:" << task->getId() << " " << task->getDataSize();;
-        }
 
-        // This was allocated in the main thread.
-        delete [] task->getData();
+            // Decompress and check sizes
+            if (chunk->miau()) {
+                dataSet->addChunk(chunk);
+            } else {
+                delete chunk;
+                BOOST_LOG_TRIVIAL(error) << "Could not decompress chunk! " << "Task:" << task->getId() << " "
+                                         << task->getDataSize();;
+            }
+
+            // This was allocated in the main thread.
+            delete[] task->getData();
+        }
     }
     
     return 0;
@@ -101,26 +99,26 @@ FTDCParser::open(std::string file_path) {
 }
 
 int
-FTDCParser::parseFiles(std::string file_paths,  bool metadata, bool metric_names) {
+FTDCParser::parseFiles(std::string file_paths, const bool onlyMetadata, const bool onlyMetricNames, const bool lazyParsing) {
     std::vector<std::string> vector_files;
 
     boost::split(vector_files, file_paths, boost::is_any_of(","));
 
-    auto ret = parseFiles(&vector_files,metadata,metric_names);
+    auto ret = parseFiles(&vector_files, onlyMetadata, onlyMetricNames, lazyParsing);
     return ret;
 }
 
 int
-FTDCParser::parseFiles(std::vector<std::string> const *file_paths,   const bool only_metadata, const bool only_metric_names) {
+FTDCParser::parseFiles(std::vector<std::string> const *filePaths, const bool onlyMetadata, const bool onlyMetricNames, const bool lazyParsing) {
     bson_reader_t *reader;
     const bson_t *pBsonChunk;
 
     double date_time_before, date_time_after, date_time_delta;
 
     namespace logging = boost::log;
-    logging::core::get()->set_filter(logging::trivial::severity >  logging::trivial::info);
+    logging::core::get()->set_filter(logging::trivial::severity >  logging::trivial::debug);
 
-    for (auto file_name : *file_paths) {
+    for (auto file_name : *filePaths) {
         BOOST_LOG_TRIVIAL(info) << "File: " << file_name;
 
         reader = this->open(file_name);
@@ -131,6 +129,8 @@ FTDCParser::parseFiles(std::vector<std::string> const *file_paths,   const bool
         bool at_EOF = false;
         unsigned int chunkCount = 0;
         bool parsing_values = false;
+        uint64_t first_id = Dataset::INVALID_TIMESTAMP;
+        uint64_t current_id = Dataset::INVALID_TIMESTAMP;
 
         while ((pBsonChunk = bson_reader_read(reader, &at_EOF))) {
 
@@ -138,11 +138,10 @@ FTDCParser::parseFiles(std::vector<std::string> const *file_paths,   const bool
             if (!parsing_values) {
                 parseInfoChunk(pBsonChunk);
                 parsing_values = true;
-                if (only_metadata) break;
+                if (onlyMetadata) break;
             }
             else {
 
-                int64_t id;
                 bson_iter_t iter;
 
                 if (bson_iter_init(&iter, pBsonChunk)) {
@@ -157,27 +156,31 @@ FTDCParser::parseFiles(std::vector<std::string> const *file_paths,   const bool
                             // the memory pointed to by data is managed internally. Better make a copy
                             uint8_t *bin_data = new uint8_t [bin_size];
                             memcpy(bin_data, data, bin_size);
-                            parserTasks.push(bin_data, bin_size, id);
+                            parserTasks.push(bin_data, bin_size, current_id);
 
                         } else if (BSON_ITER_HOLDS_DATE_TIME(&iter)) {
-                            id = bson_iter_date_time(&iter);
+
+                            current_id = bson_iter_date_time(&iter);
+                            if (first_id == Dataset::INVALID_TIMESTAMP) first_id = current_id;
                         } else if (BSON_ITER_HOLDS_INT32(&iter)) {
                             ; // type = bson_iter_int32(&iter);
                         }
                     }
                 }
                 
-                if (only_metric_names)
+                if (onlyMetricNames)
                     break;
             }
             ++chunkCount;
         }
 
+        if (lazyParsing)  dataSet.setLazyParsingFlag();
+
         // Thread pool
-        size_t numThreads =  boost::thread::hardware_concurrency();
+        size_t numThreads = boost::thread::hardware_concurrency();
         boost::thread_group threads;
         for (size_t i = 0; i < numThreads; ++i)
-            threads.add_thread( new boost::thread(ParserTaskConsumerThread, &parserTasks, &dataSet));
+            threads.add_thread(new boost::thread(ParserTaskConsumerThread, &parserTasks, &dataSet));
         // Wait for threads to finish
         threads.join_all();
 
@@ -189,9 +192,8 @@ FTDCParser::parseFiles(std::vector<std::string> const *file_paths,   const bool
         date_time_after = double_time_of_day();
         date_time_delta = BSON_MAX ((double) (date_time_after - date_time_before), 0.000001);
 
-
         // This is the end of a file being parsed.
-        dataSet.FileParsed();
+        dataSet.FileParsed(file_name.c_str(), first_id, current_id, chunkCount);
 
         BOOST_LOG_TRIVIAL(info) << "File parsed in " << date_time_delta
                  << " secs. There are " << dataSet.getChunkCount() << " chunks, "
@@ -207,7 +209,7 @@ FTDCParser::parseFiles(std::vector<std::string> const *file_paths,   const bool
                                 << "  integral unshared stack size" << usage.ru_isrss;
 
 
-        if (only_metric_names) {
+        if (onlyMetricNames) {
             std::vector<std::string> names;
             dataSet.getMetricNames(names);
             int i=0;
@@ -245,6 +247,8 @@ FTDCParser::getMetricsNames() {
 }
 
 Dataset::MetricsPtr
-FTDCParser::getMetric(std::string name, long start, long end) {
+FTDCParser::getMetric(std::string name, size_t start, size_t end) {
      return dataSet.getMetric(name, start, end);
 }
+
+
