@@ -9,8 +9,10 @@
 #include <vector>
 #include <map>
 #include <boost/thread/mutex.hpp>
+#include <SampleLocation.h>
 #include "Chunk.h"
 #include "MergerTasksList.h"
+#include "FileParsedData.h"
 
 class Dataset {
 
@@ -18,7 +20,7 @@ public:
     typedef std::vector<uint64_t> Metrics;
     typedef std::vector<uint64_t>* MetricsPtr;
 
-    Dataset() : samplesInDataset(0),  metricNames(0) {};
+    Dataset() : samplesInDataset(0),  metricNames(0), lazyParsing(0) {};
 
     void addChunk(Chunk *pChunk);
     size_t getChunkCount() { return chunkVector.size(); }
@@ -27,17 +29,27 @@ public:
     size_t getMetricNames(std::vector< std::string> & metricNames);
     [[nodiscard]] size_t getMetricLength() const { return samplesInDataset; }
 
-    MetricsPtr getMetric(std::string   metricName, long start, long end);
+    MetricsPtr getMetric(std::string   metricName, size_t start, size_t end);
 
     std::map<std::string, MetricsPtr> getHashMetrics() { return hashMapMetrics; }
     size_t getMetricNamesCount() { return metricNames.size(); }
     void sortChunks();
-    void FileParsed();
+    void FileParsed(const char * path, uint64_t start, uint64_t end, size_t samplesInFile);
     void   addMergedMetric(std::string   metricName, MetricsPtr data );
+    std::vector<FileParsedData*> getParsedFileInfo() {  return this->filesParsed; }
 
+    static const uint64_t FIRST_TIMESTAMP = UINT64_MAX;
+    static const uint64_t LAST_TIMESTAMP = UINT64_MAX;
+    static const uint64_t INVALID_TIMESTAMP = UINT64_MAX;
+
+    void setLazyParsingFlag() { lazyParsing = true; }
+    bool getLazyParsing() { return lazyParsing; }
 
 private:
     void releaseChunks();
+    std::vector<FileParsedData *> filesParsed;
+    SampleLocation getLocationInMetric(size_t sample, bool fromStart);
+    Dataset::MetricsPtr assembleMetricFromChunks(const std::string name,  SampleLocation startLocation, SampleLocation endLocation);
 
 private:
     std::vector<Chunk*> chunkVector;
@@ -45,6 +57,10 @@ private:
     boost::mutex mu;
     std::vector<std::string> metricNames;
     std::map<std::string,  MetricsPtr>  hashMapMetrics;
+
+    bool lazyParsing;
+
+
 };
 
 
