@@ -12,6 +12,8 @@
 #include <ParserTasksList.h>
 #include <string_view>
 #include <string>
+#include "FileParsedData.h"
+
 
 // From libbson
 #include <bson/bson.h>
@@ -21,17 +23,17 @@
 class FTDCParser    {
 public:
 
-    FTDCParser() : keepChunkStruct(false) {};
+    //FTDCParser() : keepChunkStruct(false) {};
 
     bson_reader_t* open(std::string file_path);
-    int parseFiles(std::vector<std::string> const *file_paths,  bool metadata, bool metric_names);
-    int parseFiles(std::string file_paths,  bool metadata, bool metric_names);
+    int parseFiles(std::vector<std::string> const *filePaths, bool onlyMetadata=false,  bool onlyMetricNames=false, bool lazyParsing=false);
+    int parseFiles(std::string filePaths, bool onlyMetadata=false,  bool onlyMetricNames=false, bool lazyParsing=false);
 
     int parseInfoChunk (const bson_t *bson);
     std::vector<std::string> getMetricsNamesPrefixed(std::string prefix) ;
     std::vector<std::string> getMetricsNames();
 
-    Dataset::MetricsPtr getMetric(std::string name, long start=-1, long end=-1);
+    Dataset::MetricsPtr getMetric(std::string name, size_t start=Dataset::INVALID_TIMESTAMP, size_t end=Dataset::INVALID_TIMESTAMP);
  
     size_t getMetricLength() { return dataSet.getMetricLength(); }
     std::vector<Chunk*> getChunks() { return dataSet.getChunkVector(); }
@@ -40,13 +42,16 @@ public:
 
     std::map<std::string,  Dataset::MetricsPtr> getHashMetrics() { return dataSet.getHashMetrics(); }
 
-    void keepChunkStructures(bool b) { keepChunkStruct = b; }
+    //void keepChunkStructures(bool b) { keepChunkStruct = b; }
+
+    std::vector<FileParsedData*> getParsedFileInfo() {return dataSet.getParsedFileInfo(); }
+
 
 private:
      ParserTasksList parserTasks;
      Dataset dataSet;
      std::vector<std::string> metadata;
-     bool keepChunkStruct;
+     //bool keepChunkStruct;
 };
 
 
