@@ -16,6 +16,8 @@ typedef std::vector<std::string> MetricNames;
 
 #define FALSE 0
 
+static const size_t INVALID_TIMESTAMP =  Dataset::INVALID_TIMESTAMP;
+
 #define STRINGIFY(x) #x
 #define MACRO_STRINGIFY(x) STRINGIFY(x)
 
@@ -53,8 +55,6 @@ struct ParserClass {
     std::vector<std::string> fileList;
     MetricNames metric_names;
     Metrics timestamps;
-    uint64_t sample_count;
-
 
     explicit ParserClass() {
         pParser = new FTDCParser();
@@ -63,14 +63,14 @@ struct ParserClass {
     int parseFile(std::string file) {
 
         fileList.emplace_back(file);
-        int n = pParser->parseFiles(&fileList, false, false);
+        int n = pParser->parseFiles(&fileList, false, false, true);
 
         if (n == 0) {
             // Timestamps, metric names, and metadata as fields in python
             metric_names = pParser->getMetricsNames();
             metadata = pParser->getMetadata();
-            timestamps = pParser->getMetric("start", 0, 0);
-            sample_count = pParser->getMetricLength();
+            timestamps = pParser->getMetric("start");
+            //sample_count = pParser->getMetricLength();
         }
         return n;
     }
@@ -84,15 +84,13 @@ struct ParserClass {
 
             // Not really necessary.
             std::sort(fileList.begin(), fileList.end());
-            int n = pParser->parseFiles(&fileList, false, false);
+            int n = pParser->parseFiles(&fileList, false, false, true);
 
             if (n == 0) {
                 // metric names and metadata as fields in python
                 metric_names = pParser->getMetricsNames();
                 metadata = pParser->getMetadata();
-                timestamps = pParser->getMetric("start", 0, 0);
-
-                sample_count = pParser->getMetricLength();
+                timestamps = pParser->getMetric("start");
             }
             return n;
 
@@ -100,35 +98,38 @@ struct ParserClass {
         return -1;
     }
     Metrics  get_timestamps() {
-        return pParser->getMetric((std::string &) "start", 0, 0);
+        return pParser->getMetric((std::string &) "start");
     }
-    Metrics  getMetric(std::string metricName, long start = -1, long end = -1) {
+    Metrics  getMetric(std::string metricName, size_t start = Dataset::INVALID_TIMESTAMP, size_t end = Dataset::INVALID_TIMESTAMP) {
         return pParser->getMetric(metricName, start, end);
     }
     uint64_t getMetricSampleCount() {
         return pParser->getMetricLength();
     }
-    py::array_t<unsigned long>  getMetricAsNumpyArray(std::string metricName, long start = -1, long end = -1) {
+    py::array_t<unsigned long>  getMetricAsNumpyArray(std::string metricName, size_t start = Dataset::INVALID_TIMESTAMP, size_t end = Dataset::INVALID_TIMESTAMP) {
         auto m = pParser->getMetric(metricName, start, end);
         return as_pyarray(m);
     }
-    std::vector<py::array_t<unsigned long>> getMetricListAsNumpyArray(std::vector<std::string> metricNames, long start = -1, long end = -1) {
+    std::vector<py::array_t<unsigned long>> getMetricListAsNumpyArray(std::vector<std::string> metricNames, size_t start = Dataset::INVALID_TIMESTAMP, size_t end = Dataset::INVALID_TIMESTAMP) {
         std::vector<py::array_t<unsigned long>> metricList;
 
         for(auto name : metricNames) {
-            auto element = as_pyarray(pParser->getMetric(name, 0, 0));
+            auto element = as_pyarray(pParser->getMetric(name, start, end));
             metricList.emplace_back(element);
         }
         return metricList;
     }
-    py::array_t<uint64_t>  getMetricListAsNumpyMatrix(std::vector<std::string> metricNames, bool transposed= FALSE) {
+    py::array_t<uint64_t>  getMetricListAsNumpyMatrix(std::vector<std::string> metricNames, bool transposed=FALSE, size_t start = Dataset::INVALID_TIMESTAMP, size_t end = Dataset::INVALID_TIMESTAMP) {
 
         if (transposed) {
             py::array_t<uint64_t, py::array::f_style> a;
+
+            //TODO: When start and end are NOT INVALID_TIMESTAMP, the second parameter is not correct.
             a.resize({(int) metricNames.size(), (int) pParser->getMetricLength()});
+
             auto r = a.mutable_unchecked();
             for (py::ssize_t i = 0; i < r.shape(0); i++) {
-                auto element = pParser->getMetric(metricNames[i], 0, 0);
+                auto element = pParser->getMetric(metricNames[i], start, end );
                 for (py::ssize_t j = 0; j < r.shape(1); j++)
                     r(j, i) = element->at(j);
             }
@@ -137,18 +138,18 @@ struct ParserClass {
         else {
             py::array_t<uint64_t, py::array::c_style> a;
 
+            //TODO: See above
             a.resize({(int) metricNames.size(), (int) pParser->getMetricLength()});
+
             auto r = a.mutable_unchecked();
             for (py::ssize_t i = 0; i < r.shape(0); i++) {
-                auto element = pParser->getMetric(metricNames[i], 0, 0);
+                auto element = pParser->getMetric(metricNames[i], start, end );
                 for (py::ssize_t j = 0; j < r.shape(1); j++)
                     r(i, j) = element->at(j);
             }
             return a;
         }
     }
-
-
 };
 
 
@@ -185,18 +186,32 @@ PYBIND11_MODULE(_core, m) {
         //.def("__repr__",   [](const ParserClass &p) {return "<Parser class object>." ;})
         .def("parse_dir", &ParserClass::parseDir)
         .def("parse_file", &ParserClass::parseFile)
-        .def("get_metric", &ParserClass::getMetric)
+        .def("get_metric", &ParserClass::getMetric,
+             "Returns a list of values from the metrics, using starting and ending timestamps if specified",
+             py::arg("metricName"),
+             py::arg("start") = INVALID_TIMESTAMP,
+             py::arg("end") = INVALID_TIMESTAMP)
         .def("get_timestamps", &ParserClass::get_timestamps)
         .def("get_metric_sample_count", &ParserClass::getMetricSampleCount)
         .def_readonly("metric_names", &ParserClass::metric_names)
         .def_readonly("metadata", &ParserClass::metadata)
         .def_readonly("timestamps", &ParserClass::timestamps)
-        .def_readonly("sample_count", &ParserClass::sample_count)
-        .def("get_metric_numpy", &ParserClass::getMetricAsNumpyArray)
-        .def("get_metrics_list_numpy", &ParserClass::getMetricListAsNumpyArray)
+        .def("get_metric_numpy", &ParserClass::getMetricAsNumpyArray,
+             "Returns a metric as a numpy array.",
+             py::arg("metricName"),
+             py::arg("start") = INVALID_TIMESTAMP,
+             py::arg("end") = INVALID_TIMESTAMP)
+        .def("get_metrics_list_numpy", &ParserClass::getMetricListAsNumpyArray,
+             "Returns a list of metrics as numpy arrays.",
+             py::arg("metricNames"),
+             py::arg("start") = INVALID_TIMESTAMP,
+             py::arg("end") = INVALID_TIMESTAMP)
         .def("get_metrics_list_numpy_matrix", &ParserClass::getMetricListAsNumpyMatrix,
-             "Returns a matrix of metrics",
-             py::arg("metricNames"), py::arg("transposed") = FALSE)
+             "Returns a matrix of metrics.",
+             py::arg("metricNames"),
+             py::arg("transposed") = FALSE,
+             py::arg("start") = INVALID_TIMESTAMP,
+             py::arg("end") = INVALID_TIMESTAMP)
         ;
 
 
