@@ -14,19 +14,19 @@
 #include <FTDCParser.h>
 #include <Chunk.h>
 #include <iostream>
-#include <boost/log/trivial.hpp>
-#include <boost/log/expressions.hpp>
 #include <boost/thread.hpp>
 
 #include <filesystem>
 
-static const char *DATA_TEST_FILE_NAME = "/Users/jorge.imperial/CLionProjects/mongo_ftdc/tests/cpp_tests/metrics.data";
-static const char *CSV__TEST_FILE_NAME = "/Users/jorge.imperial/CLionProjects/mongo_ftdc/tests/cpp_tests/first.csv";
-static const char *DATA_TEST_DIR = "/Users/jorge.imperial/CLionProjects/mongo_ftdc/tests/diagnostic.data_40/";
+// Run in the test directory
+static const char *DATA_TEST_FILE_NAME = "./cpp_tests/metrics.data";
+static const char *CSV__TEST_FILE_NAME = "./cpp_tests/first.csv";
+static const char *DATA_TEST_DIR = "./diagnostic.data_40/";
 
 
 int ParserTaskConsumerThread(ParserTasksList *parserTasks, Dataset *dataSet);
 
+static
 std::vector<ChunkMetric *>
 readMetricsFromCSV() {
 
@@ -304,7 +304,7 @@ BOOST_AUTO_TEST_SUITE(ftdc_basic_suite)
         auto dataSize = pChunk->getUncompressedSize();
         BOOST_CHECK(dataSize);
 
-        // We construct the metrics. This are name and first value only since deltas have not been read.
+        // We construct the metrics. These are name and first value only since deltas have not been read.
         int metrics = pChunk->ConstructMetrics(data);
         BOOST_CHECK_EQUAL(metrics, pChunk->getMetricsCount());
 
@@ -348,16 +348,14 @@ BOOST_AUTO_TEST_SUITE(ftdc_basic_suite)
         // Create parser
         auto parser = new FTDCParser();
 
-        parser->keepChunkStructures(true);  // Do not release Chunks after parsing is done.
+        //parser->keepChunkStructures(true);  // Do not release Chunks after parsing is done.
 
         std::vector<std::string> files;
 
         files.emplace_back(DATA_TEST_FILE_NAME);
-        namespace logging = boost::log;
 
-        logging::core::get()->set_filter(logging::trivial::severity >= logging::trivial::debug);
         auto status = parser->parseFiles(&files, false, false);
-        logging::core::get()->set_filter(logging::trivial::severity >= logging::trivial::debug);
+
         BOOST_CHECK_EQUAL(status, 0);
         auto chunkVector = parser->getChunks();
 
@@ -382,12 +380,9 @@ BOOST_AUTO_TEST_SUITE(ftdc_basic_suite)
         for (auto &&fileInPath: std::filesystem::directory_iterator(DATA_TEST_DIR))
             fileList.push_back(fileInPath.path().string());
 
-        namespace logging = boost::log;
+        //parser->keepChunkStructures(true);
 
-        parser->keepChunkStructures(true);
-        logging::core::get()->set_filter(logging::trivial::severity >= logging::trivial::info);
         auto status = parser->parseFiles(&fileList, false, false);
-        logging::core::get()->set_filter(logging::trivial::severity >= logging::trivial::debug);
         BOOST_CHECK_EQUAL(status, 0);
 
         auto chunkVector = parser->getChunks();
@@ -408,25 +403,22 @@ BOOST_AUTO_TEST_SUITE(ftdc_basic_suite)
         // Create parser
         auto parser = new FTDCParser();
 
-        namespace logging = boost::log;
-
         std::vector<std::string> fileList;
         for (auto &&fileInPath: std::filesystem::directory_iterator(DATA_TEST_DIR))
             fileList.push_back(fileInPath.path().string());
 
-        logging::core::get()->set_filter(logging::trivial::severity >= logging::trivial::info);
         auto status = parser->parseFiles(&fileList, false, false);
-        logging::core::get()->set_filter(logging::trivial::severity >= logging::trivial::debug);
+
         BOOST_CHECK_EQUAL(status, 0);
 
         auto ts = parser->getMetric("start");
-        auto end = parser->getMetric("end");
+
         uint64_t prevTimestamp = (*ts)[0] / 1000;
-        uint64_t thisTimestamp, endTimeStamp;
+        uint64_t thisTimestamp;
         for (int i = 1; i < ts->size(); ++i) {
 
             thisTimestamp = (*ts)[i] / 1000;
-            endTimeStamp = ts->at(i);
+
             if (prevTimestamp + 1 != thisTimestamp)
                 BOOST_CHECK_EQUAL(prevTimestamp + 1, thisTimestamp);
 
@@ -447,10 +439,7 @@ BOOST_AUTO_TEST_SUITE(ftdc_basic_suite)
         for (auto &&fileInPath: std::filesystem::directory_iterator(DATA_TEST_DIR))
             fileList.push_back(fileInPath.path().string());
 
-        namespace logging = boost::log;
-        logging::core::get()->set_filter(logging::trivial::severity >= logging::trivial::info);
         auto status = parser->parseFiles(&fileList, false, false);
-        logging::core::get()->set_filter(logging::trivial::severity >= logging::trivial::debug);
         BOOST_CHECK_EQUAL(status, 0);
 
         auto metricNames = parser->getMetricsNames();
@@ -463,7 +452,7 @@ BOOST_AUTO_TEST_SUITE(ftdc_basic_suite)
         BOOST_TEST_MESSAGE(msg);
     }
 
-    BOOST_AUTO_TEST_CASE(matrix) {     //
+    BOOST_AUTO_TEST_CASE(matrix_full) {     //
         // Create parser
         auto parser = new FTDCParser();
 
@@ -471,21 +460,19 @@ BOOST_AUTO_TEST_SUITE(ftdc_basic_suite)
         for (auto &&fileInPath: std::filesystem::directory_iterator(DATA_TEST_DIR))
             fileList.push_back(fileInPath.path().string());
 
-        namespace logging = boost::log;
-        logging::core::get()->set_filter(logging::trivial::severity >= logging::trivial::info);
         auto status = parser->parseFiles(&fileList, false, false);
-        logging::core::get()->set_filter(logging::trivial::severity >= logging::trivial::debug);
+
         BOOST_CHECK_EQUAL(status, 0);
 
         auto metricNames = parser->getMetricsNames();
         auto msg = str(boost::format("Metric count is %1%") % metricNames.size());
         BOOST_TEST_MESSAGE(msg);
 
-        auto ts = parser->getMetric("start", 0, 0);
-        auto m1 = parser->getMetric(metricNames[1], 0, 0);
-        auto m2 = parser->getMetric(metricNames[3], 0, 0);
-        auto m3 = parser->getMetric(metricNames[5], 0, 0);
-        auto m4 = parser->getMetric(metricNames[7], 0, 0);
+        auto ts = parser->getMetric("start");
+        auto m1 = parser->getMetric(metricNames[1]);
+        auto m2 = parser->getMetric(metricNames[3]);
+        auto m3 = parser->getMetric(metricNames[5]);
+        auto m4 = parser->getMetric(metricNames[7]);
 
         msg = str( boost::format("Timestamp has %1% elements\n%2%:%3%\n%4%:%5%\n%6%:%7%\n%8%:%9%\n") % ts->size() %
                 metricNames[1] % m1->size() %
@@ -496,5 +483,242 @@ BOOST_AUTO_TEST_SUITE(ftdc_basic_suite)
     }
 
 
+    BOOST_AUTO_TEST_CASE(timestamps) {     //
+        // Create parser
+        auto parser = new FTDCParser();
+
+        std::vector<std::string> fileList;
+        for (auto &&fileInPath: std::filesystem::directory_iterator(DATA_TEST_DIR))
+            fileList.push_back(fileInPath.path().string());
+
+        auto status = parser->parseFiles(&fileList, false, false);
+
+        BOOST_CHECK_EQUAL(status, 0);
+
+        auto metricNames = parser->getMetricsNames();
+
+        auto ts = parser->getMetric("start");
+
+        auto half_ts = (*ts)[ts->size()/2];
+
+        auto first_half  = parser->getMetric("start", Dataset::FIRST_TIMESTAMP, half_ts);
+        auto second_half = parser->getMetric("start", half_ts, Dataset::LAST_TIMESTAMP);
+        auto first_half_size = first_half->size();
+        auto second_half_size = second_half->size();
+
+        auto msg = str(boost::format("Samples in start metric is %1%. First half is %2%, second half is %3%")
+                % ts->size() % first_half_size % second_half_size);
+        BOOST_TEST_MESSAGE(msg);
+
+        BOOST_CHECK_GT(first_half_size, 0);
+        BOOST_CHECK_GT(second_half_size, 0);
+
+        BOOST_CHECK_EQUAL(ts->size(), first_half_size+second_half_size);
+
+
+        // Test case: all in one chunk:
+        auto ten_more = (*ts)[(ts->size()/2) + 10];
+        auto ten_elements = parser->getMetric("start", half_ts, ten_more );
+
+        BOOST_CHECK_EQUAL(ten_elements->size(), 10);
+
+        // Test case: across two chunks
+        auto four_hundred =  (*ts)[(ts->size()/2) + 400];
+        auto four_hundred_elements = parser->getMetric("start", half_ts, four_hundred );
+
+        BOOST_CHECK_EQUAL(four_hundred_elements->size(), 400);
+    }
+
+
+    BOOST_AUTO_TEST_CASE(matrix_with_timestamps) {     //
+        // Create parser
+        auto parser = new FTDCParser();
+
+        std::vector<std::string> fileList;
+        for (auto &&fileInPath: std::filesystem::directory_iterator(DATA_TEST_DIR))
+            fileList.push_back(fileInPath.path().string());
+
+        auto status = parser->parseFiles(&fileList, false, false);
+
+        BOOST_CHECK_EQUAL(status, 0);
+
+        auto metricNames = parser->getMetricsNames();
+        auto msg = str(boost::format("Metric count is %1%") % metricNames.size());
+        BOOST_TEST_MESSAGE(msg);
+
+        auto ts = parser->getMetric("start");
+
+        auto start_ts = (*ts)[ts->size()/2];
+        auto end_ts = (*ts)[ts->size()/2 + 1100];
+
+        auto m1 = parser->getMetric(metricNames[1], start_ts, end_ts);
+        auto m2 = parser->getMetric(metricNames[3], start_ts, end_ts);
+        auto m3 = parser->getMetric(metricNames[5], start_ts, end_ts);
+        auto m4 = parser->getMetric(metricNames[7], start_ts, end_ts);
+
+        msg = str( boost::format("Start timestamp %1%, end timestamp %2%.  Elements\n%3%:%4%\n%5%:%6%\n%7%:%8%\n%9%:%10%\n") %
+                 start_ts % end_ts %
+                   metricNames[1] % m1->size() %
+                   metricNames[3] % m2->size() %
+                   metricNames[5] % m3->size() %
+                   metricNames[7] % m4->size()
+                   );
+        BOOST_TEST_MESSAGE(msg);
+    }
+
+    BOOST_AUTO_TEST_CASE(file_info) {     //
+        // Create parser
+        auto parser = new FTDCParser();
+
+        std::vector<std::string> fileList;
+        for (auto &&fileInPath: std::filesystem::directory_iterator(DATA_TEST_DIR))
+            fileList.push_back(fileInPath.path().string());
+
+        auto status = parser->parseFiles(&fileList, false, false);
+
+        BOOST_CHECK_EQUAL(status, 0);
+
+        auto fi = parser->getParsedFileInfo();
+        for (auto f : fi) {
+
+            auto msg = str( boost::format("File %1%: samples %2%.  Start timestamp: %3%. End timestamp: %4%") %
+                               f->getFileAbsolute() %
+                               f->getSamplesCount() %
+                               f->getStart() %
+                               f->getEnd()
+            );
+            BOOST_TEST_MESSAGE(msg);
+
+        }
+
+    }
+
+
+    // - - - -
+
+    BOOST_AUTO_TEST_CASE(lazy_parser) {     //
+        // Create parser
+        auto parser = new FTDCParser();
+
+        std::vector<std::string> fileList;
+        for (auto &&fileInPath: std::filesystem::directory_iterator(DATA_TEST_DIR))
+            fileList.push_back(fileInPath.path().string());
+
+        auto status = parser->parseFiles(&fileList, false, false, true);
+
+        BOOST_CHECK_EQUAL(status, 0);
+
+        auto fi = parser->getParsedFileInfo();
+        for (auto f : fi) {
+
+            auto msg = str( boost::format("File %1%: samples %2%.  Start timestamp: %3%. End timestamp: %4%") %
+                            f->getFileAbsolute() %
+                            f->getSamplesCount() %
+                            f->getStart() %
+                            f->getEnd()
+            );
+            BOOST_TEST_MESSAGE(msg);
+        }
+
+        auto chunkVector = parser->getChunks();
+        auto pChunk = chunkVector[0];
+
+        // We construct the metrics. These are name and first value only since deltas have not been read.
+        pChunk->miau();
+    }
+
+
+    BOOST_AUTO_TEST_CASE(lazy_parser_pudding) {     //
+        // Create parser
+        auto parser = new FTDCParser();
+
+        std::vector<std::string> fileList;
+        for (auto &&fileInPath: std::filesystem::directory_iterator(DATA_TEST_DIR))
+            fileList.push_back(fileInPath.path().string());
+
+        auto status = parser->parseFiles(&fileList, false, false, true);
+
+        BOOST_CHECK_EQUAL(status, 0);
+
+        auto fi = parser->getParsedFileInfo();
+        for (auto f : fi) {
+
+            auto msg = str( boost::format("File %1%: samples %2%.  Start timestamp: %3%. End timestamp: %4%") %
+                            f->getFileAbsolute() %
+                            f->getSamplesCount() %
+                            f->getStart() %
+                            f->getEnd()
+            );
+            BOOST_TEST_MESSAGE(msg);
+        }
+
+
+        auto metricNames = parser->getMetricsNames();
+        BOOST_CHECK_GT(metricNames.size(), 0);
+
+        auto ts = parser->getMetric("start");
+
+        auto half_ts = (*ts)[ts->size()/2];
+
+        auto first_half  = parser->getMetric("start", Dataset::FIRST_TIMESTAMP, half_ts);
+        auto second_half = parser->getMetric("start", half_ts, Dataset::LAST_TIMESTAMP);
+        auto first_half_size = first_half->size();
+        auto second_half_size = second_half->size();
+
+        auto msg = str(boost::format("Sample count in 'start' metric is %1%. First half is %2%, second half is %3%")
+                       % ts->size() % first_half_size % second_half_size);
+        BOOST_TEST_MESSAGE(msg);
+
+        // Test case: all in one chunk:
+        auto ten_more = (*ts)[(ts->size()/2) + 10];
+        auto ten_elements = parser->getMetric("start", half_ts, ten_more );
+
+        BOOST_CHECK_EQUAL(ten_elements->size(), 10);
+
+        // Test case: across two chunks
+        auto four_hundred =  (*ts)[(ts->size()/2) + 400];
+        auto four_hundred_elements = parser->getMetric("start", half_ts, four_hundred );
+
+        BOOST_CHECK_EQUAL(four_hundred_elements->size(), 400);
+    }
+
+
+    BOOST_AUTO_TEST_CASE(matrix_with_timestamps_lazy) {     //
+        // Create parser
+        auto parser = new FTDCParser();
+
+        std::vector<std::string> fileList;
+        for (auto &&fileInPath: std::filesystem::directory_iterator(DATA_TEST_DIR))
+            fileList.push_back(fileInPath.path().string());
+
+        auto status = parser->parseFiles(&fileList, false, false, true);
+
+        BOOST_CHECK_EQUAL(status, 0);
+
+        auto metricNames = parser->getMetricsNames();
+        auto msg = str(boost::format("Metric count is %1%") % metricNames.size());
+        BOOST_TEST_MESSAGE(msg);
+
+        auto ts = parser->getMetric("start");
+
+        auto start_ts = (*ts)[ts->size()/2];
+        auto end_ts = (*ts)[ts->size()/2 + 1100];
+
+        auto m1 = parser->getMetric(metricNames[1], start_ts, end_ts);
+        auto m2 = parser->getMetric(metricNames[3], start_ts, end_ts);
+        auto m3 = parser->getMetric(metricNames[5], start_ts, end_ts);
+        auto m4 = parser->getMetric(metricNames[7], start_ts, end_ts);
+
+        msg = str( boost::format("Start timestamp %1%, end timestamp %2%.  Elements\n%3%:%4%\n%5%:%6%\n%7%:%8%\n%9%:%10%\n") %
+                   start_ts % end_ts %
+                   metricNames[1] % m1->size() %
+                   metricNames[3] % m2->size() %
+                   metricNames[5] % m3->size() %
+                   metricNames[7] % m4->size()
+        );
+        BOOST_TEST_MESSAGE(msg);
+    }
+
+
 BOOST_AUTO_TEST_SUITE_END()
 
